<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 27/10/14
 * Time: 16:56
 */

namespace Folha\Logic\ImportacaoLoteEvento;

use \Geral\Entities\Estruturam;

use \Geral\Entities\Estruturah;

use \Folha\Entities\GpeVinculoh;

use \Folha\Entities\GpeVinculom;

use \Folha\Entities\FpaGozoferias;

use \Folha\Entities\GpeAltesitufunc;

use \Folha\Entities\GpeSitufunch;

use \Folha\Entities\GpeAlteocupacao;

use \Folha\Entities\GpeOcupacaoh;


class Exportacao
{
    const ERRO  = 1111;
    const AVISO = 1110;

    const MSG_FALTA_VINCULO     = '';
    const MSG_NAO_EXISTE_ESTRUT = 'Não existe a Estrutura informada';
    const MSG_NENHUMA_ESTRUTURA = 'Nenhuma Estrutura foi informado';

    const NMFERIAS = 'Férias';



    /**
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrestrutsind
     * @param array $eventos
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @throws \Exception
     */
    public function __construct( $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $eventos, $dtmescompetencia, $nrorg)
    {
        try {
            $cjEventos = $this->verificarInformacoes($nrestrutlegal, $nrestrutgeren, $nrestrutgeren, $eventos, $nrorg);
            $objVinculosh = $this->getVinculos($nrestrutlegal, $nrestrutgeren, $nrestrutsind, $dtmescompetencia, $nrorg);
            $infos = $this->getInfoVinculos($objVinculosh, $dtmescompetencia, $nrorg);
            $file = new ManipulacaoExcel();
            foreach($infos as $info){
                $file->gravar($info, $cjEventos, $nrorg);
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }

    }

    /**
     * CSU003 - Verifica a integridade da informação.
     *
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrestrutsind
     * @param array $eventos
     * @param int $nrorg
     *
     * @return array
     * @throws \Exception
     */
    private function verificarInformacoes( $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $eventos, $nrorg)
    {
        try {
            if (!empty($nrestrutlegal) || !empty($nrestrutgeren) || !empty($nrestrutsind)) {
                $this->verificaEstrutura($nrestrutlegal, $nrorg);
                $this->verificaEstrutura($nrestrutgeren, $nrorg);
                $this->verificaEstrutura($nrestrutsind, $nrorg);
                $cjEventos = $this->verificaEventos($eventos, $nrorg);
            } else {
                throw new \Exception(self::MSG_NENHUMA_ESTRUTURA);
            }

            if ($cjEventos) {
                return $cjEventos;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU003a - Verifica se a estrutura existe.
     *
     * @param $nrestrutura
     * @param $nrorg
     *
     * @return boolean
     * @throws \Exception
     */
    private function verificaEstrutura( $nrestrutura, $nrorg)
    {
        try {
            if ($nrestrutura) {
                $params = array('P_NRORG' => $nrorg,
                    'P_NRESTRUTURAM' => $nrestrutura);
                $where = 'NRORG = :P_NRORG AND NRESTRUTURAM = :P_NRESTRUTURAM';
                $object = Estruturam::getObjectByWhere($where, $params);
            }else{
                return false;
            }

            if (!$object) {
                throw new \Exception(self::MSG_NAO_EXISTE_ESTRUT, self::ERRO);
            }else{
                return true;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * @todo CSU003b - Verifica se a estrutura gerencial pertence a estrutura legal
     *
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrorg
     *
     * @throws \Exception
     */
    private function verificaRelacaoEstrutura( $nrestrutlegal, $nrestrutgeren, $nrorg)
    {}

    /**
     * CSU003c - Verifica a consistência dos eventos capturados.
     * @param array $eventos
     *
     * @return array
     * @throws \Exception
     */
    private function verificaEventos( $eventos)
    {
        try {
            $cjeventos = array();
            foreach ($eventos as $evento) {
                $cjeventos[] = array('CODIGO' => $evento['nreventom'],
                                     'EVENTO' => $evento['nmeventoh']);
            }
            return $cjeventos;
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004 - Busca os vínculos
     *
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrestrutsind
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @return array
     * @throws \Exception
     */
    private function getVinculos( $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $dtmescompetencia, $nrorg)
    {
        try {
            $where = array();
            if (!empty($nrestrutlegal)) {
                $where[] = 'NRESTRUTLEGAL = ' . $nrestrutlegal;
            }
            if (!empty($nrestrutgeren)) {
                $where[] = 'NRESTRUTGEREN = ' . $nrestrutgeren;
            }
            if (!empty($nrestrutsind)) {
                $where[] = 'NRESTRUTSIND = ' . $nrestrutsind;
            }

            $where = implode(' AND ', $where);
            $objs = $this->getVinculosH($where, $dtmescompetencia, $nrorg);

            if($objs){
                return $objs;
            }else{
                return false;
            }

        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004a - Busca vínculos relacionados ao filtros de estruturas
     *
     * @param String $where
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @return bool|GpeVinculoh
     * @throws \Exception
     */
    private function getVinculosH($where, $dtmescompetencia, $nrorg)
    {
        try {
            $params = array('P_NRORG' => $nrorg,
                            'P_DTMESCOMPETENC' => $dtmescompetencia);

            $where = "$where AND NRORG = :P_NRORG AND NRVINCULOH IN (SELECT NRVINCULOH FROM GPE_VINCULOH INNER JOIN GPE_VINCULOM ON GPE_VINCULOM.NRORG = GPE_VINCULOH.NRORG AND GPE_VINCULOM.NRVINCULOM = GPE_VINCULOH.NRVINCULOM WHERE TRUNC(GPE_VINCULOM.DTADMISSAOVINC, 'DD') <= :P_DTMESCOMPETENC AND (TRUNC(GPE_VINCULOM.DTRESCISAOVINC, 'DD') >= :P_DTMESCOMPETENC OR GPE_VINCULOM.DTRESCISAOVINC IS NULL) AND DTMESCOMPETENC = (SELECT MAX(MAX_VINCULOH.DTMESCOMPETENC) FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND TRUNC(MAX_VINCULOH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENC AND MAX_VINCULOH.NRORG = :P_NRORG))";
            $object = GpeVinculoh::getObjectsByWhere($where, $params, 'NRESTRUTGEREN, NRVINCULOM');
            if ($object) {
                return $object;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005 - Busca informações do vínculo
     *
     * @param array $infoVinculos
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @return array
     * @throws \Exception
     */
    private function getInfoVinculos( $infoVinculos, $dtmescompetencia, $nrorg)
    {
        /**
         * @var GpeVinculoh $infoVinculo
         * @var GpeVinculom $objVinculom
         */
        try {
            $info = array();
            $nrestrutgeren = 0;

            foreach ($infoVinculos as $infoVinculo) {
                if ($nrestrutgeren != $infoVinculo->getNrestrutgeren()) {
                    $nrestrutgeren = $infoVinculo->getNrestrutgeren();
                    if (!$info[$nrestrutgeren]) {
                        $info[$nrestrutgeren] = array();
                    }
                }

                //Objeto Vínculom
                $objVinculom = $infoVinculo->getVinculom();

                //Ferias e Situação Funcional
                if (!$this->getFerias($infoVinculo->getNrvinculom(), $nrorg, $dtmescompetencia)) {
                    $nmsitufunc = $this->getSitufunc($infoVinculo->getNrvinculom(), $nrorg, $dtmescompetencia);
                } else {
                    $nmsitufunc = self::NMFERIAS;
                }

                //Cargo
                $nmcargo = $this->getCargo($infoVinculo->getNrvinculom(), $nrorg, $dtmescompetencia);

                //Construir Informação
                $info[$nrestrutgeren][] = array('ID' => $infoVinculo->getNrvinculom(),
                    'NOME' => $objVinculom->getNmvinculom(),
                    'SITUFUNC' => $nmsitufunc,
                    'ADMISSAO' => $objVinculom->getDtadmissaovinc()->format('d/m/Y'),
                    'CARGO' => $nmcargo,
                    'ESTRUTLEGAL' => $this->getNmestrutura($infoVinculo->getNrestrutlegal(), $nrorg, $dtmescompetencia),
                    'ESTRUTGEREN' => $this->getNmestrutura($infoVinculo->getNrestrutgeren(), $nrorg, $dtmescompetencia),
                    'ESTRUTSINDI' => $this->getNmestrutura($infoVinculo->getNrestrutsind(), $nrorg, $dtmescompetencia));


            }

            return $info;
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005a - Verifica se há um objeto de FpaGozoferias
     *
     * @param int $nrvinculom
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return GpeVinculom|bool
     * @throws \Exception
     */
    private function getFerias( $nrvinculom, $nrorg, $dtmescompetencia)
    {
        try {
            $params = array('P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = "NRORG = :P_NRORG AND NRGOZOFERIAS = (SELECT NRGOZOFERIAS FROM FPA_GOZOFERIAS WHERE NRORG = :P_NRORG AND DTINIGOZOFERIAS = (SELECT MAX(MAX_GOZOFERIAS.DTINIGOZOFERIAS) FROM FPA_GOZOFERIAS MAX_GOZOFERIAS WHERE MAX_GOZOFERIAS.NRORG = FPA_GOZOFERIAS.NRORG AND MAX_GOZOFERIAS.NRFERIAS = FPA_GOZOFERIAS.NRFERIAS AND(MAX_GOZOFERIAS.DTINIGOZOFERIAS BETWEEN TRUNC(TO_DATE(:P_DTMESCOMPETENCIA),'MONTH') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA)) OR MAX_GOZOFERIAS.DTFIMGOZOFERIAS BETWEEN TRUNC(TO_DATE(:P_DTMESCOMPETENCIA),'MONTH') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA))) AND NRFERIAS IN (SELECT NRFERIAS FROM FPA_FERIAS WHERE NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM )))";
            $result = FpaGozoferias::getObjectsByWhere($where, $params);
            if ($result) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005b - Busca objeto de Gpe
     *
     * @param int $nrvinculom
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return GpeVinculom|bool
     * @throws \Exception
     */
    private function getSitufunc( $nrvinculom, $nrorg, $dtmescompetencia)
    {
        /**
         * @var GpeAltesitufunc $objAltesitufunc
         * @var GpeSitufunch $objSitufunc
         */
        try {
            $params = array('P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = "NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND NRALTESITUFUNC = (SELECT MAX(NRALTESITUFUNC) FROM GPE_ALTESITUFUNC WHERE GPE_ALTESITUFUNC.NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND DTINISITUFUNC = (SELECT MAX(MAX_ALTESITUFUNC.DTINISITUFUNC) FROM GPE_ALTESITUFUNC MAX_ALTESITUFUNC WHERE MAX_ALTESITUFUNC.DTINISITUFUNC <= :P_DTMESCOMPETENCIA AND MAX_ALTESITUFUNC.NRORG = GPE_ALTESITUFUNC.NRORG AND MAX_ALTESITUFUNC.NRVINCULOM = GPE_ALTESITUFUNC.NRVINCULOM ))";
            $objAltesitufunc = GpeAltesitufunc::getObjectByWhere($where, $params);

            if ($objAltesitufunc) {
                $params = array('P_NRSITUFUNCM' => $objAltesitufunc->getNrsitufuncm());
                $where = "NRSITUFUNCH IN ( SELECT NRSITUFUNCH FROM GPE_SITUFUNCH WHERE NRSITUFUNCM = :P_NRSITUFUNCM AND DTEFETIVA = (SELECT MAX(MAX_SITUFUNCH.DTEFETIVA) FROM GPE_SITUFUNCH MAX_SITUFUNCH WHERE MAX_SITUFUNCH.NRSITUFUNCM = GPE_SITUFUNCH.NRSITUFUNCM AND MAX_SITUFUNCH.NRSITUFUNCM = :P_NRSITUFUNCM))";
                $objSitufunc = GpeSitufunch::getObjectByWhere($where, $params);
            }

            if ($objSitufunc) {
                return $objSitufunc->getNmsitufunch();
            } else {
                return '';
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005c - Busca objeto de FpaGozoferias
     *
     * @param int $nrvinculom
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return GpeVinculom|bool
     * @throws \Exception
     */
    private function getCargo( $nrvinculom, $nrorg, $dtmescompetencia)
    {
        /**
         * @var GpeAlteocupacao $objAlteocup
         * @var GpeOcupacaoh $objOcupacao
         */
        try {
            $params = array('P_NRVINCULOM' => $nrvinculom,
                'P_NRORG' => $nrorg,
                'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = "  NRORG = :P_NRORG  AND NRALTEOCUPACAO IN ( SELECT NRALTEOCUPACAO FROM GPE_ALTEOCUPACAO WHERE NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND DTINIOCUPACAO = (SELECT MAX(MAX_ALTEOCUPACAO.DTINIOCUPACAO) FROM GPE_ALTEOCUPACAO MAX_ALTEOCUPACAO WHERE MAX_ALTEOCUPACAO.NRVINCULOM = GPE_ALTEOCUPACAO.NRVINCULOM AND MAX_ALTEOCUPACAO.NRORG = GPE_ALTEOCUPACAO.NRORG AND TRUNC(MAX_ALTEOCUPACAO.DTINIOCUPACAO, 'DD') <= :P_DTMESCOMPETENCIA ) )";
            $objAlteocup = GpeAlteocupacao::getObjectByWhere($where, $params);

            if ($objAlteocup) {
                $params = array('P_NROCUPACAOM' => $objAlteocup->getNrocupacaom(),
                    'P_NRORG' => $nrorg,
                    'P_DTMESCOMPETENCIA' => $dtmescompetencia);
                $where = "NRORG = :P_NRORG AND NROCUPACAOH IN ( SELECT NROCUPACAOH FROM GPE_OCUPACAOH WHERE NRORG = :P_NRORG AND NROCUPACAOM = :P_NROCUPACAOM AND DTMESCOMPETENC = (SELECT MAX(MAX_OCUPACAOH.DTMESCOMPETENC) FROM GPE_OCUPACAOH MAX_OCUPACAOH WHERE MAX_OCUPACAOH.NROCUPACAOM = GPE_OCUPACAOH.NROCUPACAOM AND MAX_OCUPACAOH.NRORG = GPE_OCUPACAOH.NRORG AND TRUNC(MAX_OCUPACAOH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENCIA))";
                $objOcupacao = GpeOcupacaoh::getObjectByWhere($where, $params);
            }

            if ($objOcupacao) {
                return $objOcupacao->getNmocupacaoh();
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }

    }

    /**
     * CSU005d - Busca objeto de GpeVinculom
     *
     * @param int $nrvinculom
     * @param int $nrorg
     *
     * @return bool
     */
    private function getVinculom( $nrvinculom, $nrorg)
    {
        try {
            $param = array('P_NRVINCULOM' => $nrvinculom,
                'P_NRORG' => $nrorg);
            $where = "NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM";
            $objVinculom = GpeVinculom::getObjectByWhere($where, $param);

            if ($objVinculom) {
                return $objVinculom;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    private function getNmestrutura( $nrestruturam, $nrorg, $dtmescompetencia)
    {
        /**
         * @var Estruturah $objEstruturam
         */
        try {
            $params = array('P_NRESTRUTURAM' => $nrestruturam,
                            'P_NRORG' => $nrorg,
                            'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = " NRORG = :P_NRORG AND NRESTRUTURAH IN ( SELECT NRESTRUTURAH FROM ESTRUTURAH WHERE NRORG = :P_NRORG AND NRESTRUTURAM = :P_NRESTRUTURAM AND DTMESCOMPETENC = (SELECT MAX(MAX_ESTRUTURAH.DTMESCOMPETENC) FROM ESTRUTURAH MAX_ESTRUTURAH WHERE MAX_ESTRUTURAH.NRORG = ESTRUTURAH.NRORG AND MAX_ESTRUTURAH.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM AND TRUNC(MAX_ESTRUTURAH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENCIA))";
            $objEstruturam = Estruturah::getObjectByWhere($where, $params);

            if ($objEstruturam) {
                return $objEstruturam->getNmrazsocestrut();
            } else {
                return '';
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }
}